<?php
/**
 * Anwiki is a multilingual content management system <http://www.anwiki.com>
 * Copyright (C) 2007-2008 Antoine Walter <http://www.anw.fr>
 * 
 * Anwiki is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 * 
 * Anwiki is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Anwiki.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * ContentClass definition.
 * @package Anwiki
 * @version $Id: class_contentclass_page.php,v 1.2 2008-09-11 22:42:02 antoine-cvs Exp $
 * @copyright 2007-2008 Antoine Walter
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License 3
 */

abstract class AnwStructuredContentFieldsContainerPage extends AnwStructuredContentFieldsContainer
{
	private $aoIndexedFields = array();
	
	const FIRSTWORDS_LENGTH=30;
	
	/*protected*/ function addContentField($oContentField)
	{
		parent::addContentField($oContentField);
		
		if ($oContentField->isIndexed())
		{
			$sFieldName = $oContentField->getName();
			$sIndexName = $oContentField->getIndexName();
			if (isset($this->aoIndexedFields[$sIndexName]))
			{
				throw new AnwUnexpectedException("Index name ".$sIndexName." already used, cannot be used for ".$sFieldName);
			}
			$this->aoIndexedFields[$sIndexName] = $sFieldName;
		}
	}
	
	function getIndexedFieldName($sUserSort)
	{
		if (isset($this->aoIndexedFields[$sUserSort]))
		{
			return $this->aoIndexedFields[$sUserSort];
		}
		throw new AnwUnexpectedException("getIndexedFieldName() : No such index :".$sUserSort);
	}
	
	function getIndexes()
	{
		return $this->aoIndexedFields;
	}
	
	final function rebuildContentFromXml($sDefaultValue)
	{
		return AnwContentPage::rebuildContentFromXml($this, $sDefaultValue);
	}
}

interface AnwCachedOutputKeyDynamic
{
	function getCachedOutputKeyDynamic();
}

abstract class AnwContentClassPage extends AnwStructuredContentFieldsContainerPage implements AnwStructuredContentClass
{
	//to override
	abstract function init();
	
	abstract function toHtml($oContent, $oPage);
	
	abstract function toFeedItem($oContent, $oPage);
	
	function onChange($oPage, $oPreviousContent=null){}
	
	function getCachedOutputExpiry(){ return AnwCache::EXPIRY_UNLIMITED; }
	
	final function __construct($sContentClassName, $bIsAddon)
	{
		$this->initComponent($sContentClassName, $bIsAddon);
	}
	
	public function getLabel()
	{
		return $this->t_("contentclass_".$this->getName()."_label");
	}
	
	protected static function debug($sMessage)
	{
		return AnwDebug::log("(AnwContentClass)".$sMessage);
	}
	
	
	
	//overridable
	function pubcall($sArg, $oContent, $oPage)
	{
		return "pubcall_undefined";
	}
	
	static function pubcallOperator($sArg, $sValue, $sLang, $asOperatorArgs=array())
	{
		switch($sArg)
		{
			case "firstwords":
				$sReturn = AnwXml::xmlGetUntranslatedTxt($sValue, false);
				
				$nFirstWordsLength = self::FIRSTWORDS_LENGTH;
				if (isset($asOperatorArgs[0])&&(intval($asOperatorArgs[0])>0))
				{
					$nFirstWordsLength = intval($asOperatorArgs[0]);
				}
				
				if (strlen($sReturn)>self::FIRSTWORDS_LENGTH)
				{
					$sReturn = AnwUtils::firstWords($sValue, $nFirstWordsLength).'...';
				}
				return $sReturn;
				break;
			case "date":
				$sReturn = Anwi18n::date($sValue, $sLang);
				return $sReturn;
				break;
			case "datetime":
				$sReturn = Anwi18n::dateTime($sValue, $sLang);
				return $sReturn;
				break;
			case "year":
				$sReturn = AnwUtils::date("Y", $sValue);
				return $sReturn;
				break;
			case "monthyear":
				$sReturn = AnwUtils::date("M Y", $sValue);
				return $sReturn;
				break;
			case "count":
				$sReturn = count($sValue);
				return $sReturn;
				break;
			case "len":
				$sReturn = strlen(trim($sValue));
				return $sReturn;
				break;
			default:
				$sReturn = AnwPlugins::vhook('contentclass_pubcalloperator_default', $sValue, $sArg);
				return $sReturn;
				break;
		}
	}
	
	function getComponentName()
	{
		 return 'contentclass_'.$this->getName();
	}
		
	static function getComponentsRootDir()
	{
		return ANWDIR_CONTENTCLASSES;
	}
	static function getComponentsDirsBegin()
	{
		return 'contentclass_';
	}
	//due to "stricts oo standards", have to duplicate this function in all components class...
	static function getComponentDir($sName)
	{
		return self::getComponentsRootDir().self::getComponentsDirsBegin().$sName.'/';
	}
	
	static function getMyComponentType()
	{
		return AnwComponent::TYPE_CONTENTCLASS;
	}
	
	static function discoverEnabledComponents()
	{
		return AnwComponent::globalCfgModulesContentClasses();
	}
	
	static function loadComponent($sName)
	{
		try
		{
			$oComponentAlreadyLoaded = self::getComponentGenericIfLoaded($sName, 'contentclass');
			return $oComponentAlreadyLoaded;
		}
		catch(AnwUnexpectedException $e){}
		
		$sFile = 'contentclass_'.$sName.'.php';
		$sDir = self::getComponentDir($sName);
		$sClassNamePrefix = 'AnwContentClassPage%%_'.$sName;
		list($classname, $bIsAddon) = self::requireCustomOrDefault($sFile, $sDir, $sClassNamePrefix);
		
		$oContentClass = new $classname($sName, $bIsAddon);
		self::registerComponentGenericLoaded($sName, 'contentclass', $oContentClass);
		
		return $oContentClass;
	}
	
	static function loadComponentInterface($sName)
	{
		$sFile = 'contentclass_'.$sName.'-interface.php';
		$sDir = self::getComponentDir($sName);
		try
		{
			list($sFileDefault, $bIsAddon) = AnwUtils::getFileDefault($sFile, $sDir);
			loadApp ($sFileDefault);
		}
		catch(AnwException $e)
		{
			//no interface found for this class
		}
	}
	
	
}

?>